/* gcc_cpp */

/*
 * Problem  :   Trees
 * Contest  :   UOI-2007 (Kremenchug)
 * Type     :   Solution
 * Date     :   March 28, 2007
 * Version  :   1.1
 * Author   :   Volodymyr Tkachuk
 * Language :   C++
 * Compiler :   GCC 3.4.2
 * Algorithm:   Author Solution (Binary search with sort)
 */
#include <fstream>
#include <algorithm>
#include <vector>
using namespace std;

//Global constants
const char* inputfile = "trees.dat";
const char* outputfile = "trees.sol";
const int MAXDISTANCE = 1000000;
const int Nmax	= 1000;

int dist[Nmax][Nmax]; //All distances table
vector<int> alldist; //All diferent distances
int N; //Number of trees

//Read input data
void ReadData(){
	ifstream in(inputfile);
	in>>N;
	for(int i = 0; i<N-1; ++i)
		for(int j = i+1; j<N; ++j){
			in>>dist[i][j];
			dist[j][i]=dist[i][j];
			alldist.push_back(dist[i][j]);
		}
	in.close();
}

//Write answer to the output
void WriteData(int answer){
	ofstream out(outputfile);
	out<<answer<<endl;
	out.close();
}

vector<int> color;
// DFS
bool Fill(int index, int C, int value){
	if (color[index]==0){
		color[index]=C;
		for(int i=0; i<N; ++i)
			if (i!=index && dist[index][i]>value && !Fill(i,3-C,value))
				return false;
		return true;
	}
	else
		return (color[index]==C);
}
//Checks if it is possible to divide all trees into two parts in such way, that all trees in same part are not far than value from each other
bool CanDivide(int value){
	color.assign(N,0);
	for(int i = 0; i<N; ++i)
		if (color[i]==0 && !Fill(i,1,value))
			return false;
	return true;
}

//Find answer using binary search
int Solve(){
	int maxd = int(alldist.size())-1;
	int mind = 0;
	if (2 == N) return 0;
	while (maxd>mind){
		int middle = (maxd+mind)/2;
		if (CanDivide(alldist[middle]))
			maxd=middle;
		else
			mind=middle+1;
	}
	return alldist[maxd];
}

int main(){
	ReadData();
	sort(alldist.begin(),alldist.end()); //Sort all distances
	alldist.erase( //Get only unique distances
		unique(
			alldist.begin(),
			alldist.end()
			),
			alldist.end()
		);
	WriteData(Solve());
	return 0;
}
